//This is a threaded interpreter with callbacks for ScoreCode.
//http://pawfal.org/index.php?page=ScoreCode

#ifndef __SCORECODE_H__
#define __SCORECODE_H__

#include <vector>
#include <list>
#include <string>

#include <pthread.h>
#include <unistd.h>

//typedefs for callbacks
typedef void (*noteOnCallback)(int);
typedef void (*noteOffCallback)(int);
typedef void (*panRightCallback)();
typedef void (*panLeftCallback)();
typedef void (*panCenterCallback)();

class ScoreCode {
public:
	ScoreCode(){m_channels = 0; setBPM(60);};
	ScoreCode(int bpm, int channels);

	void run(); //starts the interpreter in a new thread
	void stop(); //stops the interpreter
	bool isRunning();
	
	//playback stuff
	void pushPattern(int channel, std::string p);
	
	void setBPM(int bpm);
	int getBPM();
	
	void setBase(int channel, int note);
	int getBase(int channel);
	
	//channel management
	int addChannel();
	int removeChannel();
	void clearChannel(int channel);
	void clearAllChannels();
	int getNumChannels();
	
	//callback registration functions
	void registerNoteOnCallback(int channel, noteOnCallback cb);
	void registerNoteOffCallback(int channel, noteOffCallback cb);
	void registerPanRightCallback(int channel, panRightCallback cb);
	void registerPanLeftCallback(int channel, panLeftCallback cb);
	void registerPanCenterCallback(int channel, panCenterCallback cb);

private:
	static void* startInt(void *arg);
	void interpret();
	int getNote(int channel);
	void setNote(int channel, int note);

	//state variables
	pthread_t m_intThread;
	pthread_mutex_t m_score_lock;
	pthread_mutex_t m_bpm_lock;
	bool m_running;
	useconds_t m_beat_length;
	int m_beats_per_minute;
	int m_channels;
	std::vector<int> m_note;
	std::vector<int> m_basenote;
	std::vector<std::list<int> > m_notestack;
	
	std::vector<std::list<char> > m_score; //the actual score
	
	//callbacks
	std::vector<noteOnCallback> m_noteOnCb;
	std::vector<noteOffCallback> m_noteOffCb;
	std::vector<panRightCallback> m_panRightCb;
	std::vector<panLeftCallback> m_panLeftCb;
	std::vector<panCenterCallback> m_panCenterCb;
};

#endif
